﻿@customization @adoSpecific
Feature: Customization: Requrement Synchronization

Rule: Synchronizes feature files as requirement work items

@tc:1299
Scenario: Creates a new work item for the requirement represented by the feature file
	Given there is a remote server project
	And the synchronizer is configured as
		| setting                                                              | value                                  |
		| customizations/requirementSynchronization/enabled                    | true                                   |
		| customizations/requirementSynchronization/requirements[0]/targetType | User Story                             |
		| customizations/requirementSynchronization/requirements[0]/condition  | $sourceFile ~ **/*.requirement.feature |
		| customizations/requirementSynchronization/requirements[0]/tagPrefix  | req                                    |
	And there is a feature file "Sample1.requirement.feature" in the local repository
		"""
		@sampleReqTag @otherReqTag
		Feature: Sample1 requirement

		This is the detailed description of the requirement
		"""
	When the local repository is synchronized with push
	Then a new Product Backlog Item "Sample1 requirement" should be created with
		| field | value                    |
		| Tags  | sampleReqTag,otherReqTag |
	And a tag "@req:[id-of-new-pbi]" should be added to the Feature in the local repository

@tc:1300
Scenario: Synchronize changes of an existing requirement to the linked Work Item
	Given there is a remote server project
	And the synchronizer is configured as
		| setting                                                              | value                      |
		| customizations/requirementSynchronization/enabled                    | true                       |
		| customizations/requirementSynchronization/requirements[0]/targetType | User Story                 |
		| customizations/requirementSynchronization/requirements[0]/condition  | $sourceFile ~ **/*.feature |
		| customizations/requirementSynchronization/requirements[0]/tagPrefix  | req                        |
	And there is an existing synchronized feature file that has been changed to:
		"""
		@req:[id-of-pbi]
		@sampleReqTag
		Feature: Updated requirement
		"""
	When the local repository is synchronized with push
	Then the Product Backlog Item title should be updated to "Updated requirement"
	And the Product Backlog Item should still have the following tags: "sampleReqTag"
	And the feature file in the local repository should not change

Rule: Scenarios can be linked to the synchronized requirement work item

@tc:1301
Scenario Outline: Scenario is linked to the requirement
	Given there is a remote server project with a Product Backlog Item "Sample requirement"
	And the synchronizer is configured as
		| setting                                                              | value                                  |
		| customizations/requirementSynchronization/enabled                    | true                                   |
		| customizations/requirementSynchronization/requirements[0]/targetType | User Story                             |
		| customizations/requirementSynchronization/requirements[0]/condition  | $sourceFile ~ **/*.requirement.feature |
		| customizations/requirementSynchronization/requirements[0]/tagPrefix  | req-id                                 |
		| synchronization/links[]/tagPrefix                                    | req-tests                              |
	And there is a feature file "<scenario file>" in the local repository
		"""
		Feature: Sample feature

		@req-tests:[id-of-pbi]
		Scenario: Sample scenario
			When I do something
		"""
	And there is a feature file "<requirement file>" in the local repository
		"""
		@req-id:[id-of-pbi]
		Feature: Sample requirement
		"""
	When the local repository is synchronized with push
	Then a new Test Case "Scenario: Sample scenario" should be created in the remote server
	And the Test Case should be linked to the Product Backlog Item as '[tests-reference-type]'
	And the local repository should be up-to-date afterwards
Examples: 
	| description                 | requirement file      | scenario file |
	| scenario processed first    | B.requirement.feature | A.feature     |
	| requirement processed first | A.requirement.feature | B.feature     |

@tc:1302
@bypass-integration
Scenario: Scenario linked to a requirement is updated using filter condition
	Given there is a remote server project with a Product Backlog Item "Sample requirement"
	And the synchronizer is configured as
		| setting                                                              | value                                  |
		| customizations/requirementSynchronization/enabled                    | true                                   |
		| customizations/requirementSynchronization/requirements[0]/targetType | User Story                             |
		| customizations/requirementSynchronization/requirements[0]/condition  | $sourceFile ~ **/*.requirement.feature |
		| customizations/requirementSynchronization/requirements[0]/tagPrefix  | req-id                                 |
		| synchronization/links[]/tagPrefix                                    | req-tests                              |
	And there is an existing synchronized feature file "Sample1.requirement.feature"
		"""
		@req-id:[id-of-pbi]
		Feature: Sample1 requirement

		@req-tests:[id-of-pbi] @mytag
		Scenario: Sample scenario
			When I do something
		"""
	When the feature file "Sample1.requirement.feature" is updated to:
		"""
		@req-id:[id-of-pbi]
		Feature: Sample1 requirement

		@tc:[id-of-test-case]
		@req-tests:[id-of-pbi] @mytag
		Scenario: Sample scenario UPDATED
			When I do something
		"""
	When the local repository is synchronized with push with the filter option "@mytag"
	Then the Test Case title should be updated to "Scenario: Sample scenario UPDATED"
	And the Test Case should be linked to the Product Backlog Item
	And the local repository should be up-to-date afterwards

@tc:1303
@bypass-integration
Scenario: Requirement linked to a scenario is updated using filter condition
	Given there is a remote server project with a Product Backlog Item "Sample requirement"
	And the synchronizer is configured as
		| setting                                                              | value                                  |
		| customizations/requirementSynchronization/enabled                    | true                                   |
		| customizations/requirementSynchronization/requirements[0]/targetType | User Story                             |
		| customizations/requirementSynchronization/requirements[0]/condition  | $sourceFile ~ **/*.requirement.feature |
		| customizations/requirementSynchronization/requirements[0]/tagPrefix  | req-id                                 |
		| synchronization/links[]/tagPrefix                                    | req-tests                              |
	And there is an existing synchronized scenario
		"""
		@req-tests:[id-of-pbi]
		Scenario: Sample scenario
			When I do something
		"""
	And there is an existing synchronized feature file "Sample1.requirement.feature"
		"""
		@req-id:[id-of-pbi] @mytag
		Feature: Sample1 requirement
		"""
	When the feature file "Sample1.requirement.feature" is updated to:
		"""
		@req-id:[id-of-pbi] @mytag
		Feature: Sample1 requirement UPDATED
		"""
	When the local repository is synchronized with push with the filter option "@mytag"
	Then the Product Backlog Item title should be updated to "Sample1 requirement UPDATED"
	And the Test Case should be linked to the Product Backlog Item
	And the local repository should be up-to-date afterwards

Rule: Scenarios of a requirement feature are linked to the requirement work item

Background: 
	Given there is a remote server project
	And the synchronizer is configured as
		| setting                                                                      | value                      |
		| customizations/requirementSynchronization/enabled                            | true                       |
		| customizations/requirementSynchronization/requirements[0]/targetType         | User Story                 |
		| customizations/requirementSynchronization/requirements[0]/condition          | $sourceFile ~ **/*.feature |
		| customizations/requirementSynchronization/requirements[0]/tagPrefix          | req                        |
		| customizations/requirementSynchronization/requirements[0]/linkContainerTests | true                       |

@tc:1304
Scenario: New requrement feature has scenarios
	Given there is a feature file in the local repository
		"""
		Feature: Sample requirement

		Scenario: Sample scenario
			When I do something
		"""
	When the local repository is synchronized with push
	Then a new Product Backlog Item "Sample requirement" should be created
	And a new Test Case "Scenario: Sample scenario" should be created in the remote server
	And the Test Case should be linked to the Product Backlog Item
	And the local repository should be up-to-date afterwards

@tc:1305
Scenario: Scenario is added to a requirement feature
	Given there is an existing synchronized feature file that has been changed to:
		"""
		@req:[id-of-pbi]
		Feature: Updated requirement

		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I do something

		Scenario: New scenario
			When I do something
		"""
	When the local repository is synchronized with push
	Then a new Test Case "Scenario: New scenario" should be created in the remote server
	And the Test Case should be linked to the Product Backlog Item
	And the local repository should be up-to-date afterwards

Rule: Scenarios in the folder and sub-folders of requirement feature are linked to the requirement work item

#TODO[req]: A new feature is needed in 'local' to mark the "main" files of the folders, that are processed first

Background: 
	Given there is a remote server project
	And the synchronizer is configured as
		| setting                                                                   | value                                  |
		| customizations/requirementSynchronization/enabled                         | true                                   |
		| customizations/requirementSynchronization/requirements[0]/targetType      | User Story                             |
		| customizations/requirementSynchronization/requirements[0]/condition       | $sourceFile ~ **/*.requirement.feature |
		| customizations/requirementSynchronization/requirements[0]/tagPrefix       | req                                    |
		| customizations/requirementSynchronization/requirements[0]/linkFolderTests | true                                   |

@tc:1306
Scenario: There are scenarios in the requirement feature's folder
	Given there is a feature file "Folder1/A.requirement.feature" in the local repository
		"""
		Feature: A requirement
		"""
	And there is a feature file "Folder1/B.feature" in the local repository
		"""
		Feature: B feature
		Scenario: B scenario
			When I do something
		"""
	And there is a feature file "Folder1/SubFolder/C.feature" in the local repository
		"""
		Feature: C feature
		Scenario: C scenario
			When I do something
		"""
	And there is a feature file "D.feature" in the local repository
		"""
		Feature: D feature (not in a folder of A)
		Scenario: D scenario
			When I do something
		"""
	When the local repository is synchronized with push
	Then a new Product Backlog Item "A requirement" should be created
	And a new Test Case "Scenario: B scenario" should be created in the remote server
	And the Test Case should be linked to the Product Backlog Item
	And a new Test Case "Scenario: C scenario" should be created in the remote server
	And the Test Case should be linked to the Product Backlog Item
	And a new Test Case "Scenario: D scenario" should be created in the remote server
	And the Test Case should not be linked to the Product Backlog Item
	And the local repository should be up-to-date afterwards

@bypass-integration
Rule: Supports branch-tagging

#TODO[req]: allow linking to requirement on the actual branch

@tc:1307
Scenario: A new requirement work item is created on an update of a branch
	Given there is a remote server project with a Product Backlog Item
	And the synchronizer is configured as
		| setting                                                                      | value                                  |
		| customizations/requirementSynchronization/enabled                            | true                                   |
		| customizations/requirementSynchronization/requirements[0]/targetType         | User Story                             |
		| customizations/requirementSynchronization/requirements[0]/condition          | $sourceFile ~ **/*.requirement.feature |
		| customizations/requirementSynchronization/requirements[0]/tagPrefix          | req-id                                 |
	And there is an existing synchronized feature file "Sample1.requirement.feature"
		"""
		@req-id:[id-of-pbi]
		Feature: Sample1 requirement
		"""
	#TODO[req]: Add unchanged
	And the synchronizer is configured as
		| setting                                                                   | value          |
		| customizations/branchTag/enabled                                          | true           |
		| customizations/branchTag/prefix                                           | tc-branch1     |
		| customizations/requirementSynchronization/requirements[0]/branchTagPrefix | req-id-branch1 |
	When the feature file "Sample1.requirement.feature" is updated to:
		"""
		@req-id:[id-of-pbi]
		Feature: Sample1 requirement UPDATED
		"""
	And a feature file "Sample3.requirement.feature" is added to the local repository
		"""
		Feature: Sample3 requirement
		"""
	When the local repository is synchronized with push
	Then a new Product Backlog Item "Sample1 requirement UPDATED" should be created
	And a tag "@req-id-branch1:[id-of-new-pbi]" should be added to the Feature "Sample1.requirement.feature" in the local repository
	And a new Product Backlog Item "Sample3 requirement" should be created
	And a tag "@req-id-branch1:[id-of-new-pbi]" should be added to the Feature "Sample3.requirement.feature" in the local repository
	And the local repository should be up-to-date afterwards

@tc:1308
Scenario: Automatically linked scenarios are relinked to the branched requirement
	Given there is a remote server project with a Product Backlog Item
	And the synchronizer is configured as
		| setting                                                                      | value                                  |
		| customizations/requirementSynchronization/enabled                            | true                                   |
		| customizations/requirementSynchronization/requirements[0]/targetType         | User Story                             |
		| customizations/requirementSynchronization/requirements[0]/condition          | $sourceFile ~ **/*.requirement.feature |
		| customizations/requirementSynchronization/requirements[0]/tagPrefix          | req-id                                 |
		| customizations/requirementSynchronization/requirements[0]/linkContainerTests | true                                   |
	And there is an existing synchronized feature file "Sample1.requirement.feature"
		"""
		@req-id:[id-of-pbi]
		Feature: Sample1 requirement

		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I do something
		"""
	And the synchronizer is configured as
		| setting                                                                   | value          |
		| customizations/branchTag/enabled                                          | true           |
		| customizations/branchTag/prefix                                           | tc-branch1     |
		| customizations/requirementSynchronization/requirements[0]/branchTagPrefix | req-id-branch1 |
	When the feature file "Sample1.requirement.feature" is updated to:
		"""
		@req-id:[id-of-pbi]
		Feature: Sample1 requirement UPDATED

		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I do something
		"""
	When the local repository is synchronized with push
	#TODO[req]: Temporary fix: multiple sync iteration might be needed
	When the local repository is synchronized with push
	Then a new Product Backlog Item "Sample1 requirement UPDATED" should be created
	And a tag "@req-id-branch1:[id-of-new-pbi]" should be added to the Feature "Sample1.requirement.feature" in the local repository
	And the local repository should be up-to-date afterwards

@tc:1309
Scenario: Branched automatically linked scenario triggers a new branched requirement
	Given there is a remote server project with a Product Backlog Item
	And the synchronizer is configured as
		| setting                                                                      | value                                  |
		| customizations/requirementSynchronization/enabled                            | true                                   |
		| customizations/requirementSynchronization/requirements[0]/targetType         | User Story                             |
		| customizations/requirementSynchronization/requirements[0]/condition          | $sourceFile ~ **/*.requirement.feature |
		| customizations/requirementSynchronization/requirements[0]/tagPrefix          | req-id                                 |
		| customizations/requirementSynchronization/requirements[0]/linkContainerTests | true                                   |
	And there is an existing synchronized feature file "Sample1.requirement.feature"
		"""
		@req-id:[id-of-pbi]
		Feature: Sample1 requirement

		@tc:[id-of-test-case]
		Scenario: Sample scenario
			When I do something
		"""
	And the synchronizer is configured as
		| setting                                                                   | value          |
		| customizations/branchTag/enabled                                          | true           |
		| customizations/branchTag/prefix                                           | tc-branch1     |
		| customizations/requirementSynchronization/requirements[0]/branchTagPrefix | req-id-branch1 |
	When the feature file "Sample1.requirement.feature" is updated to:
		"""
		@req-id:[id-of-pbi]
		Feature: Sample1 requirement

		@tc:[id-of-test-case]
		Scenario: Sample scenario UPDATED
			When I do something
		"""
	When the local repository is synchronized with push
	#TODO[req]: Temporary fix: multiple sync iteration might be needed
	When the local repository is synchronized with push
	Then a new Test Case "Scenario: Sample scenario UPDATED" should be created in the remote server
	And the Test Case should not be linked to the Product Backlog Item
	Then a new Product Backlog Item "Sample1 requirement" should be created
	And the Test Case should be linked to the Product Backlog Item
	And the local repository should be up-to-date afterwards

@bypass-integration
Rule: Requirements can be linked to other work items

@tc:1310
Scenario: A requirement is linked to a Bug, using 'related' reference type by default
	Given there is a remote server project with a Bug
	And the synchronizer is configured as
		| setting                                                              | value                                  |
		| synchronization/links[0]/tagPrefix                                   | bug                                    |
		| customizations/requirementSynchronization/enabled                    | true                                   |
		| customizations/requirementSynchronization/requirements[0]/targetType | User Story                             |
		| customizations/requirementSynchronization/requirements[0]/condition  | $sourceFile ~ **/*.requirement.feature |
		| customizations/requirementSynchronization/requirements[0]/tagPrefix  | req-id                                 |
	And there is a feature file "Sample1.requirement.feature" in the local repository
		"""
		@bug:[id-of-bug]
		Feature: Sample1 requirement
		"""
	When the local repository is synchronized with push
	Then a new Product Backlog Item "Sample1 requirement" should be created
	And the Product Backlog Item should be linked to the Bug as '[related-reference-type]'

@tc:1311
Scenario: A requirement is linked to a Bug, using specified 'related' reference type
	Given there is a remote server project with a Bug
	And the synchronizer is configured as
		| setting                                                                         | value                                  |
		| synchronization/links[0]/tagPrefix                                              | bug                                    |
		| synchronization/links[0]/relationship                                           | tests                                  |
		| customizations/requirementSynchronization/enabled                               | true                                   |
		| customizations/requirementSynchronization/requirements[0]/targetType            | User Story                             |
		| customizations/requirementSynchronization/requirements[0]/condition             | $sourceFile ~ **/*.requirement.feature |
		| customizations/requirementSynchronization/requirements[0]/tagPrefix             | req-id                                 |
		| customizations/requirementSynchronization/requirements[0]/links[0]/tagPrefix    | bug                                    |
		| customizations/requirementSynchronization/requirements[0]/links[0]/relationship | related                                |
	And there is a feature file "Sample1.requirement.feature" in the local repository
		"""
		@bug:[id-of-bug]
		Feature: Sample1 requirement
		"""
	When the local repository is synchronized with push
	Then a new Product Backlog Item "Sample1 requirement" should be created
	And the Product Backlog Item should be linked to the Bug as '[related-reference-type]'

@bypass-integration
Rule: Custom field updates can be specified for the requirements

@tc:1312
Scenario: A custom field is updated on a requirement
	Given there is a remote server project
	And the synchronizer is configured as
		| setting                                                                              | value                                  |
		| customizations/requirementSynchronization/enabled                                    | true                                   |
		| customizations/requirementSynchronization/requirements[0]/targetType                 | User Story                             |
		| customizations/requirementSynchronization/requirements[0]/condition                  | $sourceFile ~ **/*.requirement.feature |
		| customizations/requirementSynchronization/requirements[0]/tagPrefix                  | req-id                                 |
		| customizations/requirementSynchronization/requirements[0]/fieldUpdates/[state-field] | Active                                 |
	And there is a feature file "Sample1.requirement.feature" in the local repository
		"""
		Feature: Sample1 requirement
		"""
	When the local repository is synchronized with push
	Then a new Product Backlog Item "Sample1 requirement" should be created with
		| field         | value  |
		| [state-field] | Active |
	And the local repository should be up-to-date afterwards

@tc:1313
Scenario: Description and Acceptance Criteria is synchronized to a field using field updates
	Given there is a remote server project
	And the synchronizer is configured as
		| setting                                                                                            | value                                            |
		| customizations/requirementSynchronization/enabled                                                  | true                                             |
		| customizations/requirementSynchronization/requirements[0]/targetType                               | User Story                                       |
		| customizations/requirementSynchronization/requirements[0]/condition                                | $sourceFile ~ **/*.requirement.feature           |
		| customizations/requirementSynchronization/requirements[0]/tagPrefix                                | req-id                                           |
		| customizations/requirementSynchronization/requirements[0]/fieldUpdates/[description-field]         | {requirement-description:MarkdownToHtml}         |
		| customizations/requirementSynchronization/requirements[0]/fieldUpdates/[acceptance-criteria-field] | {requirement-acceptance-criteria:MarkdownToHtml} |
	And there is a feature file "Sample1.requirement.feature" in the local repository
		"""
		Feature: Sample1 requirement

		This is the detailed description of the *requirement*

		Rule: First rule
		This is a detailed description of *first* rule

		Rule: Second rule
		"""
	When the local repository is synchronized with push
	Then a new Product Backlog Item "Sample1 requirement" should be created with
		| field                       | value                                                                                                         |
		| [description-field]         | This is the detailed description of the <em>requirement</em>                                                  |
		| [acceptance-criteria-field] | <ul><li>First rule<br>\nThis is a detailed description of <em>first</em> rule</li>\n<li>Second rule</li></ul> |

